ALTER EXTENSION semver ADD TYPE semver;
ALTER EXTENSION semver ADD FUNCTION semver_in(cstring);
ALTER EXTENSION semver ADD function semver_out(semver);
ALTER EXTENSION semver ADD FUNCTION to_semver(text);
ALTER EXTENSION semver ADD FUNCTION semver(text);
ALTER EXTENSION semver ADD FUNCTION text(semver);
ALTER EXTENSION semver ADD FUNCTION semver(numeric);
ALTER EXTENSION semver ADD FUNCTION semver(real);
ALTER EXTENSION semver ADD FUNCTION semver(double precision);
ALTER EXTENSION semver ADD FUNCTION semver(integer);
ALTER EXTENSION semver ADD FUNCTION semver(smallint);
ALTER EXTENSION semver ADD FUNCTION semver(bigint);
ALTER EXTENSION semver ADD CAST (semver AS text);
ALTER EXTENSION semver ADD CAST (text AS semver);
ALTER EXTENSION semver ADD CAST (numeric AS semver);
ALTER EXTENSION semver ADD CAST (real AS semver);
ALTER EXTENSION semver ADD CAST (double precision AS semver);
ALTER EXTENSION semver ADD CAST (integer AS semver);
ALTER EXTENSION semver ADD CAST (smallint AS semver);
ALTER EXTENSION semver ADD CAST (bigint AS semver);
ALTER EXTENSION semver ADD FUNCTION semver_eq(semver, semver);
ALTER EXTENSION semver ADD FUNCTION semver_ne(semver, semver);
ALTER EXTENSION semver ADD FUNCTION semver_le(semver, semver);
ALTER EXTENSION semver ADD FUNCTION semver_lt(semver, semver);
ALTER EXTENSION semver ADD FUNCTION semver_ge(semver, semver);
ALTER EXTENSION semver ADD FUNCTION semver_gt(semver, semver);
ALTER EXTENSION semver ADD OPERATOR =(semver,semver);
ALTER EXTENSION semver ADD OPERATOR <>(semver,semver);
ALTER EXTENSION semver ADD OPERATOR >(semver,semver);
ALTER EXTENSION semver ADD OPERATOR >=(semver,semver);
ALTER EXTENSION semver ADD OPERATOR <(semver,semver);
ALTER EXTENSION semver ADD OPERATOR <=(semver,semver);
ALTER EXTENSION semver ADD FUNCTION semver_cmp(semver, semver);
ALTER EXTENSION semver ADD FUNCTION hash_semver(semver);
ALTER EXTENSION semver ADD OPERATOR CLASS semver_ops USING btree;
ALTER EXTENSION semver ADD OPERATOR CLASS semver_ops USING hash;
ALTER EXTENSION semver ADD FUNCTION semver_smaller(semver, semver);
ALTER EXTENSION semver ADD FUNCTION semver_larger(semver, semver);
ALTER EXTENSION semver ADD AGGREGATE min(semver);
ALTER EXTENSION semver ADD AGGREGATE max(semver);
